// BVQXPluginInterface.h : general header file for BrainVoyagerQX plugin support
//

#ifndef BVQXPLUGININTERFACE_H
#define BVQXPLUGININTERFACE_H

#include "Plugin_VMR_Header.h"
#include "Plugin_VMP_Header.h"
#include "Plugin_VTC_Header.h"
#include "Plugin_CMP_Header.h"
#include "Plugin_GLM_Header.h"
#include "Plugin_PRT_Header.h"
#include "Plugin_VOI_Header.h"
#include "Plugin_SRF_Header.h"
#include "Plugin_MTC_Header.h"
#include "Plugin_SMP_Header.h"
#include "Plugin_SSM_Header.h"
#include "Plugin_POI_Header.h"

#include "global.h"


// make sure that "MAKE_PLUGIN" is defined here or as a preprocessor definition (this is only important on Windows)
//
#define MAKE_PLUGIN

#if defined OS_WIN32
#ifdef MAKE_PLUGIN
#define PLUGIN_ACCESS __declspec(dllexport)
#else
#define PLUGIN_ACCESS __declspec(dllimport)
#endif
#else
// for all other platforms PLUGIN_ACCESS is empty (not needed)
#define PLUGIN_ACCESS
#endif


class PLUGIN_ACCESS BVQXPluginInterface
{
public:	 

	// Use this method to execute the plugin (your essential code)
	//
	virtual bool execute() = 0;

	// Use this function to check whether your plugin is ready to be used, i.e. if conditions are met
	// A VMR must be available, for example, before it can be used. At present this function is not called from QX.
	//
	virtual bool isAvailable() const = 0;
};


//
// These are typedef's for dynamic access of QX functions
//

// general functions
typedef void (*GetVersionOfBrainVoyagerQXPrototype)(int *, int *, int *);
extern GetVersionOfBrainVoyagerQXPrototype qxGetVersionOfBrainVoyagerQX;

typedef void (*GetBrainVoyagerQXPathPrototype)(char *);
extern GetBrainVoyagerQXPathPrototype qxGetBrainVoyagerQXPath;

typedef void (*GetPluginsPathPrototype)(char *);
extern GetPluginsPathPrototype qxGetPluginsPath;


// input / output
typedef void (*ShowLogTabPrototype)();
extern ShowLogTabPrototype qxShowLogTab;

typedef void (*LogTextPrototype)(const char *);
extern LogTextPrototype qxLogText;

typedef void (*MessageBoxPrototype)(const char *);
extern MessageBoxPrototype qxMessageBox;

typedef int (*YesNoMessageBoxPrototype)(const char *);
extern YesNoMessageBoxPrototype qxYesNoMessageBox;

typedef int (*GetIntegerInputPrototype)(const char *, int, int, int);
extern GetIntegerInputPrototype qxGetIntegerInput;

typedef float (*GetFloatInputPrototype)(const char *, float, float, float);
extern GetFloatInputPrototype qxGetFloatInput;

typedef int (*GetTextInputPrototype)(const char *, const char *, char *);
extern GetTextInputPrototype qxGetTextInput;

typedef int (*GetOpenFileNamePrototype)(const char *, const char *, const char *, char *);
extern GetOpenFileNamePrototype qxGetOpenFileName;

typedef int (*GetSaveFileNamePrototype)(const char *, const char *, const char *, char *);
extern GetSaveFileNamePrototype qxGetSaveFileName;


// GUI
typedef void (*UpdateActiveWindowPrototype)();
extern UpdateActiveWindowPrototype qxUpdateActiveWindow;

typedef void (*ShowBusyCursorPrototype)();
extern ShowBusyCursorPrototype qxShowBusyCursor;

typedef void (*StopBusyCursorPrototype)();
extern StopBusyCursorPrototype qxStopBusyCursor;


// VMR
typedef unsigned char * (*GetCurrentVMRPrototype)(int *, int *, int *);
extern GetCurrentVMRPrototype qxGetCurrentVMR;

typedef unsigned int (*GetMainHeaderOfCurrentVMRPrototype)(struct VMR_Header *);
extern GetMainHeaderOfCurrentVMRPrototype qxGetMainHeaderOfCurrentVMR;

typedef unsigned int (*SetMainHeaderOfCurrentVMRPrototype)(struct VMR_Header *);
extern SetMainHeaderOfCurrentVMRPrototype qxSetMainHeaderOfCurrentVMR;

typedef unsigned int (*GetScannerPositionHeaderOfCurrentVMRPrototype)(struct VMR_ScannerPosition_Header *);
extern GetScannerPositionHeaderOfCurrentVMRPrototype qxGetScannerPositionHeaderOfCurrentVMR;

typedef unsigned int (*SetScannerPositionHeaderOfCurrentVMRPrototype)(struct VMR_ScannerPosition_Header *);
extern SetScannerPositionHeaderOfCurrentVMRPrototype qxSetScannerPositionHeaderOfCurrentVMR;

typedef unsigned int (*GetPastTransformationsHeaderOfCurrentVMRPrototype)(struct VMR_PastTransformations_Header *);
extern GetPastTransformationsHeaderOfCurrentVMRPrototype qxGetPastTransformationsHeaderOfCurrentVMR;

typedef unsigned int (*SetPastTransformationsHeaderOfCurrentVMRPrototype)(struct VMR_PastTransformations_Header *);
extern SetPastTransformationsHeaderOfCurrentVMRPrototype qxSetPastTransformationsHeaderOfCurrentVMR;

typedef unsigned int (*ReframeCurrentVMRPrototype)(int, int, int);
extern ReframeCurrentVMRPrototype qxReframeCurrentVMR;

typedef int (*GetCrossPositionOfCurrentVMRPrototype)(int *, int *, int *);
extern GetCrossPositionOfCurrentVMRPrototype qxGetCrossPositionOfCurrentVMR;


// VMP
typedef int (*GetVMPsOfCurrentVMRPrototype)(struct VMPs_Header *);
extern GetVMPsOfCurrentVMRPrototype qxGetVMPsOfCurrentVMR;

typedef float * (*GetVMPOfCurrentVMRPrototype)(int, struct VMP_Header *);
extern GetVMPOfCurrentVMRPrototype qxGetVMPOfCurrentVMR;

typedef int (*SetVMPParametersOfCurrentVMRPrototype)(int, struct VMP_Header *);
extern SetVMPParametersOfCurrentVMRPrototype qxSetVMPParametersOfCurrentVMR;

typedef int (*LoadVMPsToCurrentVMRPrototype)(const char *);
extern LoadVMPsToCurrentVMRPrototype qxLoadVMPsToCurrentVMR;

typedef void (*SaveVMPsOfCurrentVMRPrototype)(const char *);
extern SaveVMPsOfCurrentVMRPrototype qxSaveVMPsOfCurrentVMR;

typedef int (*CreateVMPForCurrentVMRPrototype)(struct VMPs_Header *);
extern CreateVMPForCurrentVMRPrototype qxCreateVMPForCurrentVMR;

typedef void (*DeleteVMPsOfCurrentVMRPrototype)();
extern DeleteVMPsOfCurrentVMRPrototype qxDeleteVMPsOfCurrentVMR;

// VTC
typedef int (*LinkVTCToCurrentVMRPrototype)(const char *, int);
extern LinkVTCToCurrentVMRPrototype qxLinkVTCToCurrentVMR;

typedef short * (*GetVTCOfCurrentVMRPrototype)(struct VTC_Header *);
extern GetVTCOfCurrentVMRPrototype qxGetVTCOfCurrentVMR;

typedef int (*SetVTCParametersOfCurrentVMRPrototype)(struct VTC_Header *);
extern SetVTCParametersOfCurrentVMRPrototype qxSetVTCParametersOfCurrentVMR;

typedef int (*CreateVTCForCurrentVMRPrototype)(int, struct VTC_Header *);
extern CreateVTCForCurrentVMRPrototype qxCreateVTCForCurrentVMR;

typedef void (*SaveVTCOfCurrentVMRPrototype)(const char *);
extern SaveVTCOfCurrentVMRPrototype qxSaveVTCOfCurrentVMR;

typedef int (*ClearMultiVTCAnalysisPrototype)();
extern ClearMultiVTCAnalysisPrototype qxClearMultiVTCAnalysis;

typedef int (*AddVTCRefForMultiVTCAnalysisPrototype)(const char *);
extern AddVTCRefForMultiVTCAnalysisPrototype qxAddVTCRefForMultiVTCAnalysis;

typedef int (*GetVoxelTimeCourseOfVTCPrototype)(int , int , unsigned short *);
extern GetVoxelTimeCourseOfVTCPrototype qxGetVoxelTimeCourseOfVTC;

typedef int (*GetVoxelTimeCourseOfVTCsPrototype)(int , unsigned short *);
extern GetVoxelTimeCourseOfVTCsPrototype qxGetVoxelTimeCourseOfVTCs;

typedef int (*GetVoxelCoordsTimeCourseOfVTCPrototype)(int , int , int, int, unsigned short *);
extern GetVoxelCoordsTimeCourseOfVTCPrototype qxGetVoxelCoordsTimeCourseOfVTC;

typedef int (*GetVoxelCoordsTimeCourseOfVTCsPrototype)(int , int, int, unsigned short *);
extern GetVoxelCoordsTimeCourseOfVTCsPrototype qxGetVoxelCoordsTimeCourseOfVTCs;


// GLM
typedef float * (*GetGLMOfCurrentVMRPrototype)(struct GLM_Header *);
extern GetGLMOfCurrentVMRPrototype qxGetGLMOfCurrentVMR;


// MSK
typedef int (*LoadMaskToCurrentVMRPrototype)(const char *);
extern LoadMaskToCurrentVMRPrototype qxLoadMaskToCurrentVMR;

typedef unsigned char * (*GetMaskOfCurrentVMRPrototype)();
extern GetMaskOfCurrentVMRPrototype qxGetMaskOfCurrentVMR;


// SRF
typedef int (*GetScenePrototype)(struct SCENE_Header *);
extern GetScenePrototype qxGetScene;

typedef int (*GetCurrentSRFPrototype)(struct SRF_Header *);
extern GetCurrentSRFPrototype qxGetCurrentSRF;

typedef void (*SaveCurrentSRFPrototype)(const char *);
extern SaveCurrentSRFPrototype qxSaveCurrentSRF;

typedef int (*SetCurrentSRFPrototype)(int);
extern SetCurrentSRFPrototype qxSetCurrentSRF;


// SMP
typedef int (*GetSMPsOfCurrentSRFPrototype)(struct SMPs_Header *);
extern GetSMPsOfCurrentSRFPrototype qxGetSMPsOfCurrentSRF;

typedef float * (*GetSMPOfCurrentSRFPrototype)(int, struct SMP_Header *);
extern GetSMPOfCurrentSRFPrototype qxGetSMPOfCurrentSRF;

typedef int (*SetSMPParametersOfCurrentSRFPrototype)(int, struct SMP_Header *);
extern SetSMPParametersOfCurrentSRFPrototype qxSetSMPParametersOfCurrentSRF;

typedef int (*LoadSMPsToCurrentSRFPrototype)(const char *);
extern LoadSMPsToCurrentSRFPrototype qxLoadSMPsToCurrentSRF;

typedef void (*SaveSMPsOfCurrentSRFPrototype)(const char *);
extern SaveSMPsOfCurrentSRFPrototype qxSaveSMPsOfCurrentSRF;

typedef int (*CreateSMPForCurrentSRFPrototype)();
extern CreateSMPForCurrentSRFPrototype qxCreateSMPForCurrentSRF;

typedef void (*DeleteSMPsOfCurrentSRFPrototype)();
extern DeleteSMPsOfCurrentSRFPrototype qxDeleteSMPsOfCurrentSRF;

typedef void (*OverlaySMPsOnSRFPrototype)();
extern OverlaySMPsOnSRFPrototype qxOverlaySMPsOnSRF;


// MTC
typedef int (*LinkMTCToCurrentSRFPrototype)(const char *, int);
extern LinkMTCToCurrentSRFPrototype qxLinkMTCToCurrentSRF;

typedef int (*CreateMTCForCurrentSRFPrototype)(int);
extern CreateMTCForCurrentSRFPrototype qxCreateMTCForCurrentSRF;

typedef float ** (*GetMTCOfCurrentSRFPrototype)(struct MTC_Header *);
extern GetMTCOfCurrentSRFPrototype qxGetMTCOfCurrentSRF;

typedef int (*SetMTCParametersOfCurrentSRFPrototype)(struct MTC_Header *);
extern SetMTCParametersOfCurrentSRFPrototype qxSetMTCParametersOfCurrentSRF;

typedef void (*SaveMTCOfCurrentSRFPrototype)(const char *);
extern SaveMTCOfCurrentSRFPrototype qxSaveMTCOfCurrentSRF;


// SSM
typedef int * (*GetSSMOfCurrentSRFPrototype)(struct SSM_Header *);
extern GetSSMOfCurrentSRFPrototype qxGetSSMOfCurrentSRF;

typedef int (*LoadSSMToCurrentSRFPrototype)(const char *);
extern LoadSSMToCurrentSRFPrototype qxLoadSSMToCurrentSRF;

typedef void (*SaveSSMOfCurrentSRFPrototype)(const char *);
extern SaveSSMOfCurrentSRFPrototype qxSaveSSMOfCurrentSRF;

typedef int (*CreateSSMForCurrentSRFPrototype)();
extern CreateSSMForCurrentSRFPrototype qxCreateSSMForCurrentSRF;

typedef void (*DeleteSSMOfCurrentSRFPrototype)();
extern DeleteSSMOfCurrentSRFPrototype qxDeleteSSMOfCurrentSRF;


// CMP
typedef float ** (*GetCMPsOfCurrentVMRPrototype)(struct CMPS_Header *);
extern GetCMPsOfCurrentVMRPrototype qxGetCMPsOfCurrentVMR;

typedef int (*SetCMPsParamsDisplayRangePrototype)(struct CMPS_Header *);
extern SetCMPsParamsDisplayRangePrototype qxSetCMPsParamsDisplayRange;

typedef float * (*GetCMPOfCurrentVMRPrototype)(int, struct CMP_Header *);
extern GetCMPOfCurrentVMRPrototype qxGetCMPOfCurrentVMR;

typedef int (*SetCMPParametersOfCurrentVMRPrototype)(int, struct CMP_Header *);
extern SetCMPParametersOfCurrentVMRPrototype qxSetCMPParametersOfCurrentVMR;

typedef int (*LoadCMPsToCurrentVMRPrototype)(const char *);
extern LoadCMPsToCurrentVMRPrototype qxLoadCMPsToCurrentVMR;

typedef void (*SaveCMPsOfCurrentVMRPrototype)(const char *);
extern SaveCMPsOfCurrentVMRPrototype qxSaveCMPsOfCurrentVMR;

typedef int (*CreateCMPsForCurrentVMRPrototype)(int, int, int, struct CMPS_Header *);
extern CreateCMPsForCurrentVMRPrototype qxCreateCMPsForCurrentVMR;

typedef void (*DeleteCMPsOfCurrentVMRPrototype)();
extern DeleteCMPsOfCurrentVMRPrototype qxDeleteCMPsOfCurrentVMR;


// PRT
typedef void (*ClearProtocolPrototype)();
extern ClearProtocolPrototype qxClearProtocol;

typedef int (*LoadProtocolPrototype)(const char *);
extern LoadProtocolPrototype qxLoadProtocol;

typedef int (*SaveProtocolPrototype)(const char *);
extern SaveProtocolPrototype qxSaveProtocol;

typedef int (*GetProtocolPrototype)(struct Protocol_Header *);
extern GetProtocolPrototype qxGetProtocol;

typedef int (*SetProtocolPrototype)(struct Protocol_Header *);
extern SetProtocolPrototype qxSetProtocol;

typedef int (*GetProtocolConditionPrototype)(int, struct ProtocolCondition_Header *);
extern GetProtocolConditionPrototype qxGetProtocolCondition;

typedef int (*AddProtocolConditionPrototype)(struct ProtocolCondition_Header *);
extern AddProtocolConditionPrototype qxAddProtocolCondition;

typedef int (*AddProtocolConditionIntervalPrototype)(int, int, int);
extern AddProtocolConditionIntervalPrototype qxAddProtocolConditionInterval;


// PRED, DM, HRF
typedef int (*ClearDesignMatrixPrototype)();
extern ClearDesignMatrixPrototype qxClearDesignMatrix;

typedef int (*AddPredictorPrototype)(const char *);
extern AddPredictorPrototype qxAddPredictor;

typedef int (*SetPredictorValuesFromConditionPrototype)(const char *, int, float);
extern SetPredictorValuesFromConditionPrototype qxSetPredictorValuesFromCondition;

typedef int (*SetPredictorValuesFromConditionNamePrototype)(const char *, const char *, float);
extern SetPredictorValuesFromConditionNamePrototype qxSetPredictorValuesFromConditionName;

typedef int (*ApplyTwoGammaHRFToPredictorPrototype)(int);
extern ApplyTwoGammaHRFToPredictorPrototype qxApplyTwoGammaHRFToPredictor;

typedef int (*ApplyTwoGammaHRFToPredictorWithNamePrototype)(const char *);
extern ApplyTwoGammaHRFToPredictorWithNamePrototype qxApplyTwoGammaHRFToPredictorWithName;

typedef int (*ApplyBoyntonHRFToPredictorPrototype)(int);
extern ApplyBoyntonHRFToPredictorPrototype qxApplyBoyntonHRFToPredictor;

typedef int (*ApplyBoyntonHRFToPredictorWithNamePrototype)(const char *);
extern ApplyBoyntonHRFToPredictorWithNamePrototype qxApplyBoyntonHRFToPredictorWithName;

typedef float * (*GetPredictorPrototype)(int);
extern GetPredictorPrototype qxGetPredictor;

typedef float * (*GetPredictorByNamePrototype)(const char *);
extern GetPredictorByNamePrototype qxGetPredictorByName;

typedef int (*LoadSingleStudyGLMDesignMatrixPrototype)(const char *);
extern LoadSingleStudyGLMDesignMatrixPrototype qxLoadSingleStudyGLMDesignMatrix;

typedef int (*SaveSingleStudyGLMDesignMatrixPrototype)(const char *);
extern SaveSingleStudyGLMDesignMatrixPrototype qxSaveSingleStudyGLMDesignMatrix;


// VOI
typedef int (*GetVOIsPrototype)(struct VOIs_Header *);
extern GetVOIsPrototype qxGetVOIs;

typedef int * (*GetVOIPrototype)(int, struct VOI_Header *);
extern GetVOIPrototype qxGetVOI;


// POI
typedef int (*GetPOIsPrototype)(struct POIs_Header *);
extern GetPOIsPrototype qxGetPOIs;

typedef int * (*GetPOIPrototype)(int, struct POI_Header *);
extern GetPOIPrototype qxGetPOI;


// dialog access
typedef int (*GetDlgStringParameterPrototype)(int, const char *);
extern GetDlgStringParameterPrototype qxGetDlgStringParameter;

typedef int (*GetDlgIntegerParameterPrototype)(int);
extern GetDlgIntegerParameterPrototype qxGetDlgIntegerParameter;

typedef float (*GetDlgFloatParameterPrototype)(int);
extern GetDlgFloatParameterPrototype qxGetDlgFloatParameter;

typedef void (*LaunchICADialogPrototype)();
extern LaunchICADialogPrototype qxLaunchICADialog;


// Locally defined functions
extern bool InitQXAccess();
extern bool checkVersionGreaterEqualVersion(int MajorA, int MinorA, int SubMinorA, int MajorB, int MinorB, int SubMinorB);


#endif // BVQXPLUGININTERFACE_H
