
#include "SurfaceTool.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "colortools.h"
#include "otherfunctions.h"


SurfaceTool::SurfaceTool()
{
}


SurfaceTool::~SurfaceTool()
{
}


int FUNCTIONLIST[5] = {FUNCTION_CHANGECOLORS, FUNCTION_PRINTHDRTOLOG, FUNCTION_PRINTCOLORSTOLOG, FUNCTION_HEX2DEC, FUNCTION_WRITEHDRTOFILE};
int sizeFuncFiles = (sizeof(FUNCTIONLIST)/sizeof(int));

char *descrTxt = getFunctionDescrList("This surface tool provides the following functions:", FUNCTIONLIST, sizeFuncFiles, true, "<BR>");
static const char *pluginDescription = descrTxt;


bool SurfaceTool::execute() {

	int ret = InitQXAccess();
	if(!ret) return false;

	qxShowLogTab();
	qxLogText("\nSurfaceTool> Started...");

	// Get a pointer to the current VMR
	int DimX, DimY, DimZ;
	unsigned char *VMR = qxGetCurrentVMR(&DimX, &DimY, &DimZ);

	if(VMR == 0) {		
		qxMessageBox("This plugin requires that a VMR document is available and selected in BrainVoyager QX.");
		qxLogText( "SurfaceTool> No VMR present, action cancelled." );
		return false;
	}
	SCENE_Header *scene = new SCENE_Header;						  	
	SRF_Header *srfqx = new SRF_Header;		
	bool success;
	int dec;
	char *txt;

	qxGetScene(scene);
	if (scene->NrOfMeshes > 0) {				
		qxGetCurrentSRF(srfqx);

		char *menu = getFunctionDescrList("Please select one of the following functions:", FUNCTIONLIST, sizeFuncFiles, true, "");
		int functionnr = qxGetIntegerInput(menu, FUNCTION_CHANGECOLORS, 1, sizeFuncFiles);

		switch(functionnr) {
			case FUNCTION_CHANGECOLORS:
				success = changeMeshColorRGB(srfqx);
				break;
			case FUNCTION_PRINTHDRTOLOG:
				qxLogText(currentsrfhdr2ascii(srfqx));
				break;
			case FUNCTION_PRINTCOLORSTOLOG:
				printColorsOfRangeToLogtab(srfqx);
				break;
			case FUNCTION_HEX2DEC:
				dec = hexToDec();
				break;
			case FUNCTION_WRITEHDRTOFILE:
				txt = currentsrfhdr2ascii(srfqx);
				writecurrentsrfhdr(srfqx, txt);
				break;
			default:
				qxLogText("Process cancelled.");
		}
	} else {
		qxLogText("No surface present. Functionality will not be available.");
	}
	return true;
}

bool SurfaceTool::isAvailable() const 
{
	return this->checkPluginAvailability();
}

bool SurfaceTool::checkPluginAvailability() const 
{
	return true;
}


// THIS FUNCTION CREATES YOUR PLUGIN CLASS
//
PLUGIN_ACCESS BVQXPluginInterface *createPlugin() 
{
	return new SurfaceTool();
}

// return a name for your plugin
//
PLUGIN_ACCESS const char *getPluginName() 
{
	return "Surface tool (v0.2)";
}

// returns a descriptive string for your plugin shown in the QX "Descriptions of Plugins" window
//   it is recommended to use a static C string to describe your plugin
//   (the potentially lengthy string needs then not be created repeatedly in successive calls)
//   Note that you may use simple HTML tags to format the string, such as <b></b>, <i></i> and <br>
//
PLUGIN_ACCESS const char *getPluginDescription() 
{
	return pluginDescription;
}

// get credit for your work!
//
PLUGIN_ACCESS const char *getAuthor()
{
	return "<i>Hester Breman, Brain Innovation B.V.</i>";
}

// you may provide the name for a HTML file containing online help
//
PLUGIN_ACCESS const char *getHelpFile()
{
	return "";
}

PLUGIN_ACCESS const char *getRegistrationOfGenericComponentLoader()
{
	return ""; // only return a non-empty string if you use CMP maps (for details, see User's Guide)
}

