
#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <exception>
#include <math.h>
#include <string.h>

#include "BVQXPluginInterface.h"
#include "Plugin_SRF_Header.h"
#include "otherfunctions.h"


char *currentsrfhdr2ascii(SRF_Header *srf) {
	
	char *buf; int bufLen; 
	if (srf == NULL) return "No SRF content present.\n";
	buf = (char *)calloc(1,65534); bufLen = 0;  
	
	try {
		
		sprintf(buf, "\nSRF filename: \n%s\n", srf->MeshFileName); //free(ebuf);
		sprintf(buf+strlen(buf), "Number of vertices: %d\n", srf->NrOfVertices);
		sprintf(buf+strlen(buf), "Number of triangles: %d\n", srf->NrOfTriangles);
		sprintf(buf+strlen(buf), "Mesh center: X: %f, Y: %f, Z: %f\n", srf->MeshCenterX, srf->MeshCenterY, srf->MeshCenterZ);
		sprintf(buf+strlen(buf), "Visible: %d\n", srf->Visible);
		sprintf(buf+strlen(buf), "Mesh curvature colors: \n");
		sprintf(buf+strlen(buf), "Concave red: %f, green: %f, blue: %f\n", srf->Base1ColorB, srf->Base1ColorB, srf->Base1ColorB);		
		sprintf(buf+strlen(buf), "Convex red: %f, green: %f, blue: %f\n", srf->Base2ColorB, srf->Base2ColorB, srf->Base2ColorB);
		sprintf(buf+strlen(buf), "Mesh rotation X: %f, Y: %f, Z: %f\n", srf->MeshRotX, srf->MeshRotY, srf->MeshRotZ);
		sprintf(buf+strlen(buf), "Number of triangle strip elements: %d\nUse strips: %d\n", srf->NrOfStripElements, srf->UseTriangleStrip);
		sprintf(buf+strlen(buf), "Render mode: %d\n", srf->RenderMode);
		
		bufLen = strlen(buf);
		buf  = (char *)realloc((void *)buf, bufLen+1);
		
	} catch (...) {
		qxLogText("Error while printing current SRF header data to ascii");
		free(srf);
	}				
	
	return buf;
}

void writecurrentsrfhdr(SRF_Header *srf, char *txt) {
	
	FILE *info = fopen(exchangeExtension(srf->MeshFileName, ".txt"), "w");
	fputs(txt, info);
	fclose(info);	
	qxLogText("Header information of file saved to text.\nIt is located in the same folder as the surface file.");
}

char *exchangeExtension(char *currentFileName, char *newExtension) {
	
	int length;
	char *result;
	char *fileNameWithoutExt = removeExtension(currentFileName);
	length = strlen(fileNameWithoutExt);
	int extLength = strlen(newExtension);
	result = new char[length+extLength];
	strcpy(result, fileNameWithoutExt);
	strcat(result, newExtension);
	return result;
	
}

char *removeExtension(char *fileName){

	int ii; int strLength; int lout;
	char *outName ;
	if( fileName == NULL || (strLength=strlen(fileName)) == 0){  
		outName = strdup("''") ; return outName;
	}
	int endIndex = 0;
	lout = 4 ;                    
	for( ii=0 ; ii < strLength ; ii++ ){ 
		switch( fileName[ii] ){
		case '.': endIndex = ii;		
		default: lout++ ; break ;   
			
		}
	}
	outName = (char *)calloc(1,lout) ;    
	for( ii=0; ii < endIndex; ii++){
		outName[ii] = fileName[ii];
	}
	outName[ii]   = '\0' ;  
	return outName;
}


char *getFunctionDescrList(char *prologue, int list[], int sizelist, bool htmlend, char *epilogue) {
	
	char *descrList = new char[65536];
	sprintf(descrList, "%s%s", prologue, (htmlend ? "<P>" : "\n\n"));
	int i;
	for (i=0; i<sizelist; i++) {	
		sprintf(descrList +strlen(descrList), "%d: %s %s", (i+1), getFunctionDescr(list[i]), (htmlend ? "<BR>":"\n"));
	}
	sprintf(descrList +strlen(descrList), "%s", epilogue);
	return descrList;
}

char *getFunctionDescr(int functionnr) {

	switch(functionnr) {
		case FUNCTION_CHANGECOLORS:		return "Change colors of vertices";
		case FUNCTION_PRINTHDRTOLOG:	return "Print the header information of the current surface to the Log tab";
		case FUNCTION_PRINTCOLORSTOLOG:	return "Print a range of color values of the current surface to the Log tab"; 
		case FUNCTION_HEX2DEC:			return "Hexadecimal to decimal converter for HTML color codes";
		case FUNCTION_WRITEHDRTOFILE:	return "Write header information of current surface to a text file";
		default:						return "Unknown function";
	}	 
}



